package ru.amse.bazylevich.faeditor.fautomaton.algorithms;

import java.util.LinkedHashSet;
import java.util.Set;

import ru.amse.bazylevich.faeditor.fautomaton.IAutomaton;
import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Automaton;

public class MinimizerAutomaton {
	
	public static IAutomaton minimize(IAutomaton oldAutomaton) {
		IAutomaton automaton = new Automaton();
		Set<ITransition> reachableTransitions = new LinkedHashSet<ITransition>();
		IState initialState = oldAutomaton.getInitialState();
		if (initialState != null) {
			makeTransition(oldAutomaton.getInitialState(), reachableTransitions);
			Set<IState> reachableStates = makeReachableStates(reachableTransitions);
			reachableStates.add(initialState);
			for (IState state : reachableStates) {
				if (oldAutomaton.getInitialState() == state) {
					automaton.setInitialState(state);
				} else if (oldAutomaton.getFinalStates().contains(state)) {
					automaton.addFinalState(state);
				} else {
					automaton.addState(state);
				}
			}
		}
		return automaton;
	}
	
	private static void makeTransition(IState state, Set<ITransition> reachableTransitions) {
		for (ITransition transition : state.getTransitions()) {
			if (!reachableTransitions.contains(transition)) {
				reachableTransitions.add(transition);
				makeTransition(transition.getEnd(), reachableTransitions);
			} else {
				return;
			}
		}
	}
	
	
	private static Set<IState> makeReachableStates(Set<ITransition> set) {
		Set<IState> states = new LinkedHashSet<IState>();
		for (ITransition transition : set) {
			states.add(transition.getStart());
			states.add(transition.getEnd());
		}
		return states;
	}
}
