package ru.amse.bazylevich.faeditor.fautomaton.file;

import java.util.ArrayList;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import ru.amse.bazylevich.faeditor.fautomaton.IAutomaton;
import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Condition;
import ru.amse.bazylevich.faeditor.fautomaton.impl.State;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Transition;

public class AutomatonLoaderHandler extends DefaultHandler{
	
	private final IAutomaton myAutomaton;
	private final ArrayList<IState> myStates;
	private int myNumberOfInitialState;
	
	public AutomatonLoaderHandler(IAutomaton automaton) {
		myAutomaton = automaton;
		myStates = new ArrayList<IState>();
		myNumberOfInitialState = -1;
	}
	
	
	
	@Override
	public void startElement(String uri, String localName, String name,
			Attributes attributes) throws SAXException {
		if (name.equals("state")) {
			IState state = new State(attributes.getValue("label"));
			String type = attributes.getValue("type");
			if (attributes.getLength() == 2) {
				if (type.equals("final")) {
					myAutomaton.addFinalState(state);
				}
			} else {
				myAutomaton.addState(state);
			}
			myStates.add(state);
		} else if (name.equals("transition")) {
			IState start = myStates.get(new Integer(attributes.getValue("start")));
			ITransition transition = new Transition(start, myStates.get(new Integer(attributes.getValue("end"))), new Condition(attributes.getValue("condition")));
			start.addTransition(transition);
		} 
		else
			if (name.equals("automaton")) {
			myNumberOfInitialState = new Integer(attributes.getValue("initialstate"));
		} 
	}
	
	@Override
	public void endDocument() throws SAXException {
		if (myNumberOfInitialState != -1) {
			myAutomaton.setInitialState(myStates.get(myNumberOfInitialState));
		}
		myStates.clear();
	}

}
