package ru.amse.bazylevich.faeditor.fautomaton.impl;

import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

import ru.amse.bazylevich.faeditor.fautomaton.IAutomaton;
import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;


public class Automaton implements IAutomaton {
	
	private final Set<IState> myStates = new LinkedHashSet<IState>();
	private final Set<IState> myFinalStates = new LinkedHashSet<IState>();
	private IState myInitialState;
	
	
	public Set<IState> getStates() {
		return Collections.unmodifiableSet(myStates);
	}
	
	public IState getInitialState() {
		return myInitialState;
	}
	
	
	public Set<IState> getFinalStates() {
		return Collections.unmodifiableSet(myFinalStates);
	}
	
	public void addState(IState state) {
		myStates.add(state);
	}
	
	public void addFinalState(IState state) {
		myStates.add(state);
		myFinalStates.add(state);
	}
	
	public void setInitialState(IState state) {
		myInitialState = state;
		myStates.add(state);
	}
	
	@Override
	public void removeState(IState state) {
		for (IState startState : myStates) {
			LinkedHashSet<ITransition> delete = new LinkedHashSet<ITransition>();
			for (ITransition transition : startState.getTransitions()) {
				if (transition.getEnd() == state) {
					delete.add(transition);
				}
			}
			for (ITransition transition : delete) {
				startState.removeTransition(transition);
			}
		}
		myStates.remove(state);
		myFinalStates.remove(state);
		if (myInitialState == state) {
			myInitialState = null;
		}
		
	}
	
	@Override
	public void removeTransition(ITransition transition) {
		if (transition == null) {
			return;
		}
		for (IState state : myStates) {
			if (transition.getStart() == state) {
				state.removeTransition(transition);
				return;
			}
		}
	}
	
	
	@Override
	public void toInitialState(IState state) {
		myInitialState = state;
	}
	
	
	@Override
	public void removeFromFinalStates(IState state) {
		myFinalStates.remove(state);
	}
	
	
}
