package ru.amse.bazylevich.faeditor.main;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.LinkedHashSet;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import ru.amse.bazylevich.faeditor.fautomaton.impl.Automaton;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IAutomatonPresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.JFAutomaton;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.impl.AutomatonWhithWayDrawer;
import ru.amse.bazylevich.faeditor.ui.fautomaton.file.AutomatonPresentationLoader;
import ru.amse.bazylevich.faeditor.ui.fautomaton.file.IAutomatoPresentationnLoader;
import ru.amse.bazylevich.faeditor.ui.fautomaton.impl.AutomatonPresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.listeners.IComponentChangedListener;
import ru.amse.bazylevich.faeditor.ui.fautomaton.listeners.ISelectListener;
import ru.amse.bazylevich.faeditor.ui.fautomaton.tools.impl.AddTool;
import ru.amse.bazylevich.faeditor.ui.fautomaton.tools.impl.DefaultTool;
import ru.amse.bazylevich.faeditor.ui.interpreter.JDrawString;
import ru.amse.bazylevich.faeditor.ui.interpreter.JInterpreter;
import ru.amse.bazylevich.faeditor.ui.menuactions.DeleteAction;
import ru.amse.bazylevich.faeditor.ui.menuactions.DetermineAction;
import ru.amse.bazylevich.faeditor.ui.menuactions.MinimizeAction;
import ru.amse.bazylevich.faeditor.ui.menuactions.OpenAction;
import ru.amse.bazylevich.faeditor.ui.menuactions.SaveAction;
import ru.amse.bazylevich.faeditor.ui.menuactions.SaveAsAction;

public class MainFrame extends JFrame {
	
	private File myCurrentFile;
	private File myPreviousFile;
	private boolean myIsChanged;
	private ExitFrameAction myExitAction;
	private SaveAction mySaveAction;
	private boolean myIsAsked;
	
	public MainFrame() {
		super("Finite Automaton Editor");
		myCurrentFile = null;
		myPreviousFile = null;
		myIsChanged = false;
		myIsAsked = false;
		setSize(800, 800);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new ExitMainFrameListener());
		IAutomatonPresentation aPresentation = new AutomatonPresentation(new Automaton(), new LinkedHashSet<IStatePresentation>());
		IAutomatoPresentationnLoader loader;
		try {
			loader = new AutomatonPresentationLoader();
		} 	catch (SAXException e) {
			loader = null;
			JOptionPane.showMessageDialog(this, "Fatal error. Please restart aplication", "Error" ,JOptionPane.WARNING_MESSAGE);
		} catch (ParserConfigurationException e) {
			loader = null;
			JOptionPane.showMessageDialog(this, "Fatal error. Please restart aplication", "Error" ,JOptionPane.WARNING_MESSAGE);
		} 
		
		AutomatonWhithWayDrawer draw = new AutomatonWhithWayDrawer();
		final JFAutomaton jautomaton = new JFAutomaton(aPresentation, draw);
		jautomaton.addListener(new IComponentChangedListener() {
			@Override
			public void ComponentChanged() {
				if (getTitle().charAt(0) != '*') {
					setTitle("*" + getTitle());
					myIsChanged = true;
					myIsAsked = false;
					repaint();
				}
			}
		});
		JDrawString drawString = new JDrawString();
		drawString.setPreferredSize(new Dimension(0, 40));
		final JInterpreter jInterpreter = new JInterpreter(jautomaton, draw, drawString);
		jautomaton.addListener(new IComponentChangedListener() {
			@Override
			public void ComponentChanged() {
				jInterpreter.stop();
			}
		});
	
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		
		panel.add(jInterpreter, BorderLayout.NORTH);
		panel.add(drawString, BorderLayout.CENTER);
		
		getContentPane().add(panel, BorderLayout.NORTH);
		
		getContentPane().add(jautomaton, BorderLayout.CENTER);
	
		
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		
		JMenu file = new JMenu("File");
		menuBar.add(file);
		
		JFileChooser fileChooser =  new JFileChooser();
		mySaveAction = new SaveAction("Save", this, jautomaton, fileChooser);

		
		file.add(new AbstractAction ("New") { 
			@Override
			public void actionPerformed(ActionEvent e) {
				if (showSaveDialog(e)) {
					jautomaton.setAutomaton(new AutomatonPresentation(new Automaton(), new LinkedHashSet<IStatePresentation>()));
					if (myCurrentFile != null) {
						myPreviousFile = myCurrentFile;
					}
					myCurrentFile = null;
					myPreviousFile = null;
					jautomaton.remove(jautomaton.getLabelEditor());
					setNormalTitle();
				}
			}
		});
		file.add(new OpenAction(this, jautomaton, loader, fileChooser));
		file.add(mySaveAction);
		myExitAction = new ExitFrameAction(mySaveAction);
		SaveAsAction saveAsAction = new SaveAsAction("Save as", this, jautomaton, fileChooser);
		file.add(saveAsAction);
		file.add(myExitAction);
		JMenu edit = new JMenu("Edit");
		menuBar.add(edit);
		final DeleteAction deleteAction = new DeleteAction(jautomaton , this);
		jautomaton.addListener(new ISelectListener() {
			@Override
			public void elementsSelected() {
				deleteAction.setEnabled(true);
				
			}
			@Override
			public void elementsUnselected() {
				deleteAction.setEnabled(false);
			}
		});
		edit.add(deleteAction);
		
		JMenu transform = new JMenu("Transform");
		menuBar.add(transform);
		transform.add(new DetermineAction(jautomaton, this));
		transform.add(new MinimizeAction(jautomaton, this));
		
		
		JMenu tools = new JMenu("Tools");
		menuBar.add(tools);
		JRadioButtonMenuItem addTool = new JRadioButtonMenuItem();
		addTool.setAction(new AbstractAction("Add") {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				jautomaton.setTool(new AddTool(jautomaton));
				jautomaton.clearSelectedStates();
				jautomaton.setSelectedTransition(null);
				jautomaton.repaint();
			}
		});
		JRadioButtonMenuItem defaultTool = new JRadioButtonMenuItem();
		defaultTool.setAction(new AbstractAction("Default") {
			@Override
			public void actionPerformed(ActionEvent e) {
				jautomaton.setTool(new DefaultTool(jautomaton));
				jautomaton.repaint();
			}
		});
		defaultTool.setSelected(true);
		tools.add(defaultTool);
		tools.add(addTool);
		ButtonGroup group = new ButtonGroup();
		group.add(addTool);
		group.add(defaultTool);
		setVisible(true);

	}
	
	private class ExitMainFrameListener extends WindowAdapter {
		@Override
		public void windowClosing(WindowEvent e) {
			myExitAction.actionPerformed(null);
		}
	}
	
	private class ExitFrameAction extends AbstractAction {
		
		public ExitFrameAction(SaveAction saveAction) {
			super("Exit");
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			if (showSaveDialog(e)) { 
				System.exit(0);
			}
		}
	}
	
	public boolean showSaveDialog(ActionEvent e) {
		if (myIsChanged && !myIsAsked) {
			int res = JOptionPane.showConfirmDialog(MainFrame.this, "File has been modified. Save changes?",
					"Save file", JOptionPane.YES_NO_CANCEL_OPTION);
			if (res == JOptionPane.YES_OPTION) {
				mySaveAction.actionPerformed(e);
				myIsAsked = true;
				return true;
			} else if (res == JOptionPane.CANCEL_OPTION) {
				return false;
			}
			
		}
		myIsAsked = true;
		return true;
	}
	
	public File getCurrentFile() {
		return myCurrentFile;
	}
	
	public void setCurrentFile(File file) {
		myCurrentFile = file;
	}
	
	public File getPreviousFle() {
		return myPreviousFile;
	}
	
	public void setPreviosFile(File file) {
		myPreviousFile = file;
	}
	
	public void setNormalTitle() {
		if (myIsChanged) {
			setTitle(getTitle().substring(1));
			myIsChanged = false;
			repaint();
		}

	}
	
	public void setAsked() {
		myIsAsked = false;
	}
}
