package ru.amse.bazylevich.faeditor.ui.fautomaton;


import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JComponent;

import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.IAutomatonDrawer;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.impl.AutomatonDrawer;
import ru.amse.bazylevich.faeditor.ui.fautomaton.listeners.IComponentChangedListener;
import ru.amse.bazylevich.faeditor.ui.fautomaton.listeners.ISelectListener;
import ru.amse.bazylevich.faeditor.ui.fautomaton.tools.ITool;
import ru.amse.bazylevich.faeditor.ui.fautomaton.tools.impl.DefaultTool;


public class JFAutomaton extends JComponent {
	
	public static int MYWIDTH = 10;
	
	
	private IAutomatonPresentation myAutomatonPresentation;
	private final Set<IStatePresentation> mySelectedStates;
	private ITransition mySelectedTransition;
	private ITool myTool;
	private IAutomatonDrawer myDraw;
	private final List<IComponentChangedListener> myComponentChangeListeners;
	private final List<ISelectListener> mySelectElementsListeners;
	private final JTypeOfStateMenu myTypeOfStateChooser;
	private final JLabelEditor myEditor;
	
	
	public JFAutomaton(IAutomatonPresentation automaton, IAutomatonDrawer draw) {
		myAutomatonPresentation = automaton;
		mySelectedStates = new LinkedHashSet<IStatePresentation>();
		myTool = new DefaultTool(this);
		myDraw = draw;
		myComponentChangeListeners = new LinkedList<IComponentChangedListener>();
		mySelectElementsListeners = new LinkedList<ISelectListener>();
		myTypeOfStateChooser = new JTypeOfStateMenu(this);
		myTypeOfStateChooser.setState(null);
		myEditor = new JLabelEditor(this);
		setComponentPopupMenu(myTypeOfStateChooser);
		setLayout(null);
		setFocusable(true);
		JFAtomatonListener listener = new JFAtomatonListener();
		addMouseListener(listener);
		addMouseMotionListener(listener);
	}
	
	public void setAutomaton(IAutomatonPresentation automaton) {
		myAutomatonPresentation = automaton;
		repaint();
	}
	
	public IAutomatonPresentation getAutomatonPresentation() {
		
		return myAutomatonPresentation;
	}
	
	public void setSelectedTransition(ITransition transition) {
		mySelectedTransition = transition;
	}
	
	public ITransition getSelectedTransition() {
		return mySelectedTransition;
	}
	
	public void setSelectedStates(Set<IStatePresentation> selectedStates) {
		mySelectedStates.clear();
		mySelectedStates.addAll(selectedStates);
	}
	
	public Set<IStatePresentation> getSelectedStates() {
		return mySelectedStates;
	}
	
	public void addSelectedState(IStatePresentation sPresentation) {
		mySelectedStates.clear();
		mySelectedStates.add(sPresentation);
	}
	
	public void clearSelectedStates() {
		mySelectedStates.clear();
	}
	
	public IAutomatonDrawer getDraw() {
		return myDraw;
	}
	
	public JTypeOfStateMenu getTypeOfStateChooser() {
		return myTypeOfStateChooser;
	}
	
	public void setTool(ITool tool) {
		myTool = tool;
	}
	
	public JLabelEditor getLabelEditor() {
		return myEditor;
	}
	
	public void addListener(IComponentChangedListener listener) {
		myComponentChangeListeners.add(listener);
	}
	
	public void addListener(ISelectListener listener) {
		mySelectElementsListeners.add(listener);
	}
	
	public void fireComponentChanged() {
		for (IComponentChangedListener listener : myComponentChangeListeners) {
			listener.ComponentChanged();
		}
	}

	
	public void fireElementsSelected() {
		for (ISelectListener listener : mySelectElementsListeners) {
			listener.elementsSelected();
		}
	}
	
	public void fireElementsUnselected() {
		for (ISelectListener listener : mySelectElementsListeners) {
			listener.elementsUnselected();
		}

	}
	
	
	public void removeSelectedElements() {
		for (IStatePresentation sPresentation : mySelectedStates) {
			myAutomatonPresentation.removeStatePresentation(sPresentation);
		}
		clearSelectedStates();
		myAutomatonPresentation.getAutomaton().removeTransition(mySelectedTransition);
		mySelectedTransition = null;
		repaint();
	}
	
	public void paintComponent(Graphics gc) {
		if (myAutomatonPresentation == null) {
			return;
		}
		((Graphics2D) gc).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		for (IStatePresentation statePresentation : myAutomatonPresentation.getStatesPresentations()) {
			IState state = statePresentation.getState();
			myDraw.drawTransitions(statePresentation, gc, myAutomatonPresentation);
			if (state == myAutomatonPresentation.getAutomaton().getInitialState()) {
				myDraw.drawInitialState(statePresentation, gc);
			} else if (myAutomatonPresentation.getAutomaton().getFinalStates().contains(state)) {
				myDraw.drawFinalState(statePresentation, gc);
			} else {
				myDraw.drawState(statePresentation, gc);
			}
			if (mySelectedStates.contains(statePresentation)) {
				gc.drawRect(statePresentation.getX() - AutomatonDrawer.MYSTATEWIDTH / 2, statePresentation.getY() - AutomatonDrawer.MYSTATEHEIGHT / 2, AutomatonDrawer.MYSTATEWIDTH, AutomatonDrawer.MYSTATEHEIGHT);
			}
		}
		myTool.finishDrawing(gc);
	}
	
	
	private class JFAtomatonListener extends MouseAdapter implements MouseMotionListener {
		
		public void mouseClicked(MouseEvent mEvent) {
			myTool.mouseClicked(mEvent);
			repaint();
		}
			
		public void mousePressed(MouseEvent mEvent) {
			requestFocusInWindow();
			myTool.mousePressed(mEvent);
		}

		
		public void mouseDragged(MouseEvent mEvent) {
			myTool.mouseDragged(mEvent);
			repaint();
		}
		
		public void mouseReleased(MouseEvent mEvent) {
			myTool.mouseReleased(mEvent);
		}

		
		public void mouseMoved(MouseEvent mEvent) {
			myTool.mouseMoved(mEvent);
		}
		
	}

}
			
	


























