package ru.amse.bazylevich.faeditor.ui.fautomaton.draw.impl;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;

import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IAutomatonPresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.IAutomatonDrawer;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.util.Geometry;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.AutomatonPresentatios;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.Point;

public class AutomatonDrawer implements IAutomatonDrawer {
	
	public final static int MYSTATEWIDTH = 70;
	public final static int MYSTATEHEIGHT = 30;
	public final static int MYLENGTHARROW = 196;
	public final static double MYARROWANGLE = Math.PI / 6;
	public final static int MYLOOPHEIGHT = 36;
	
	
	protected class Angle {
		double myAngle;
		
		public Angle(double a) {
			myAngle = a;
		}
		
		public Angle() {
			this(0);
		}
	}

	
	public void drawInitialState(IStatePresentation sPresentation, Graphics gc) {
		((Graphics2D) gc).fillOval(sPresentation.getX() - MYSTATEWIDTH / 2, sPresentation.getY() - MYSTATEHEIGHT / 2,
                MYSTATEWIDTH, MYSTATEHEIGHT);
		gc.setColor(Color.WHITE);
		drawLabel(sPresentation.getState().getLabel(), sPresentation.getX(), 
                sPresentation.getY(), gc);
		gc.setColor(Color.BLACK);
	}
	
	
	public void drawState(IStatePresentation sPresentation, Graphics gc) {
		drawOval(sPresentation, gc);
		drawLabel(sPresentation.getState().getLabel(), sPresentation.getX(), 
				                                       sPresentation.getY(), gc);
	}
	
	private void drawOval(IStatePresentation sPresentation, Graphics gc) {
		gc.drawOval(sPresentation.getX() - MYSTATEWIDTH / 2, sPresentation.getY() - MYSTATEHEIGHT / 2,
                MYSTATEWIDTH, MYSTATEHEIGHT);
	}
	
	public void drawFinalState(IStatePresentation sPresentation, Graphics gc) {
		drawState(sPresentation, gc);
		gc.drawOval(sPresentation.getX() - MYSTATEWIDTH / 2 + 4, sPresentation.getY() - MYSTATEHEIGHT / 2 + 4,
                MYSTATEWIDTH - 8, MYSTATEHEIGHT - 8);
	}
	
	
	public void drawTransitions(IStatePresentation sPresentation, Graphics gc, IAutomatonPresentation aPresentation) {
		Point startPoint = new Point(sPresentation.getX(), sPresentation.getY());
		for (ITransition transition : sPresentation.getState().getTransitions()) {
			IStatePresentation endSPresentation = AutomatonPresentatios.getStatePresentation(aPresentation, transition.getEnd());
			Angle theta = new Angle();
			Point middlePoint = new Point();
			drawTransition(startPoint, sPresentation, endSPresentation, transition, theta, middlePoint, gc);
			drawCondition(transition.getCondition().toString(), middlePoint, gc, theta.myAngle);
		}
	}
	
	protected void drawTransition(Point startPoint, IStatePresentation startState, IStatePresentation endState, ITransition transition, Angle theta, Point middlePoint, Graphics gc) {
		if (startState.getState() == endState.getState()) {
			Point point = new Point(startState.getX() - MYLOOPHEIGHT / 2, startState.getY() - MYSTATEHEIGHT / 2 - MYLOOPHEIGHT);
			drawLoop(startState, transition, point, gc);
			middlePoint.setX(endState.getX());
			middlePoint.setY(endState.getY() - MYSTATEHEIGHT / 2 - MYLOOPHEIGHT);
		} else {
			Point endPoint = new Point(endState.getX(), endState.getY());
			drawEdge(startPoint, endPoint, gc);
			theta.myAngle = Geometry.calculatingAngel(startPoint, endPoint);
			middlePoint.setX((startPoint.getX() + endPoint.getX()) / 2);
			middlePoint.setY((startPoint.getY() + endPoint.getY()) / 2);
		}
	}
		
	
	
	@Override
	public void drawAddingTransition(String condition, Point startPoint, Point endPoint, Graphics gc) {
		((Graphics2D) gc).setStroke(
			       new BasicStroke(
			               1.3f,
			               BasicStroke.CAP_ROUND,
			               BasicStroke.JOIN_ROUND,
			               1.0f,
			               new float[]{3,4},
			               0
			       )
			);
		gc.drawLine(startPoint.getX(), startPoint.getY(), endPoint.getX(), endPoint.getY());
		drawArrow(startPoint, endPoint , gc);
		double theta = Geometry.calculatingAngel(startPoint, endPoint);
		Point middlePoint = new Point((startPoint.getX() + endPoint.getX()) / 2, (startPoint.getY() + endPoint.getY()) / 2);
		drawCondition(condition, middlePoint, gc, theta);
	}
	
	
	private void drawLabel(String string, int x, int y, Graphics gc) {
		int width = gc.getFontMetrics().stringWidth(string);
		if (width < MYSTATEWIDTH - 5) {
			gc.drawString(string, x - width / 2 , y + 3);
		} else {
			gc.drawString("...", x - gc.getFontMetrics().stringWidth("...") / 2 , y + 3);
		}
	}
	
	protected void drawArrow(Point start, Point end, Graphics gc) {
		
		int x1 = start.getX();
		int y1 = start.getY();
		int x2 = end.getX();
		int y2 = end.getY();
		
		x1 -= x2;
		y1 -= y2;
		double[] point = new double[2];
		if (x1 == 0) {
			if (y1 < 0) {
				point[1] = - Math.pow(MYLENGTHARROW, 0.5);
			} else if (y1 >= 0) {
				point[1] = Math.pow(MYLENGTHARROW, 0.5);
			}
		} else {
			double k = ((double) y1/ x1); 
			double root1 = Math.pow(MYLENGTHARROW / (1 + k * k), 0.5);
			if (x1 < 0) {
				point[0] = -root1;
			} else {
				point[0] = root1;
			}
			point[1] = (k * point[0]);
		}
		point[0] += x2;
		point[1] += y2;
		
		((Graphics2D) gc).rotate(MYARROWANGLE , x2, y2);
		gc.drawLine((int) (point[0] + 0.5),(int) (point[1]+ 0.5), x2, y2);
		((Graphics2D) gc).rotate(- 2 * MYARROWANGLE , x2, y2);
		gc.drawLine((int) (point[0] + 0.5),(int) (point[1]+ 0.5), x2, y2);
		((Graphics2D) gc).rotate(MYARROWANGLE , x2, y2);
	
	}
	
	protected void drawEdge(Point start, Point end, Graphics gc) {
	
		Point newStart = Geometry.computingEdgeEnd(start, end);
		Point newEnd = Geometry.computingEdgeEnd(end, start);
		
	    gc.drawLine(newStart.getX(), newStart.getY(), newEnd.getX(), newEnd.getY());
		
		drawArrow(newStart, newEnd , gc);
	}
	
	protected void drawCondition(String string, Point point, Graphics gc, double theta) {
		((Graphics2D) gc).rotate( theta, point.getX(), point.getY());
		int width = gc.getFontMetrics().stringWidth(string);
		gc.drawString(string, point.getX() - width / 2, point.getY() - 3);
		((Graphics2D) gc).rotate( - theta, point.getX(), point.getY());
	}
	
	private void drawLoop(IStatePresentation state, ITransition transition, Point point, Graphics gc) {
		gc.drawArc(point.getX(), point.getY(), MYLOOPHEIGHT, MYLOOPHEIGHT, 0, 360);
		drawArrow(new Point(state.getX() + MYLOOPHEIGHT / 4, state.getY() - MYSTATEHEIGHT - MYLOOPHEIGHT), new Point(state.getX() + MYLOOPHEIGHT / 2, state.getY() - MYSTATEHEIGHT / 2 - MYLOOPHEIGHT / 2), gc);
	}

	
}
