package ru.amse.bazylevich.faeditor.ui.fautomaton.draw.impl;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IAutomatonPresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.AutomatonPresentatios;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.Point;

public class AutomatonWhithWayDrawer extends AutomatonDrawer {
	
	private final List<ITransition> myTransitions;
	private final List<IStatePresentation> myStates;
	private String myString;
	
	public AutomatonWhithWayDrawer() {
		myTransitions = new LinkedList<ITransition>();
		myStates = new LinkedList<IStatePresentation>();
	}
	
	public void clearLists() {
		myTransitions.clear();
		myStates.clear();
	}
	
	public void addStates(IStatePresentation sPresentation) {
		myStates.add(sPresentation);
	}
	
	public void removeState(){
		myStates.remove(myStates.size() -1);
	}
	
	public void addTransition(ITransition transition) {
		myTransitions.add(transition);
	}
	
	public void removeTransition() {
		((LinkedList<ITransition>) myTransitions).removeLast();
	}
	
	public void setString(String string) {
		myString = string;
	}
	
	
	@Override
	public void drawTransitions(IStatePresentation sPresentation, Graphics gc, IAutomatonPresentation aPresentation) {
		Point startPoint = new Point(sPresentation.getX(), sPresentation.getY());
		for (ITransition transition : sPresentation.getState().getTransitions()) {
			IStatePresentation endSPresentation = AutomatonPresentatios.getStatePresentation(aPresentation, transition.getEnd());
			if (myTransitions.contains(transition)) {
				gc.setColor(Color.RED);
			}
			Angle theta = new Angle();
			Point middlePoint = new Point();
			super.drawTransition(startPoint, sPresentation, endSPresentation, transition, theta, middlePoint, gc);
			gc.setColor(Color.BLACK);
			drawCondition(transition, middlePoint, gc, theta.myAngle);
		}
	}
	
	
	@Override
	public void drawInitialState(IStatePresentation sPresentation, Graphics gc) {
		setColor(sPresentation, gc);
		super.drawInitialState(sPresentation, gc);
		gc.setColor(Color.BLACK);
	}
	
	@Override
	public void drawState(IStatePresentation sPresentation, Graphics gc) {
		setColor(sPresentation, gc);
		super.drawState(sPresentation, gc);
		gc.setColor(Color.BLACK);
	}
	
	@Override
	public void drawFinalState(IStatePresentation sPresentation, Graphics gc) {
		setColor(sPresentation, gc);
		super.drawState(sPresentation, gc);
		gc.drawOval(sPresentation.getX() - MYSTATEWIDTH / 2 + 4, sPresentation.getY() - MYSTATEHEIGHT / 2 + 4,
                MYSTATEWIDTH - 8, MYSTATEHEIGHT - 8);
		gc.setColor(Color.BLACK);
	}
	
	private void setColor(IStatePresentation sPresentation, Graphics gc) {
		if (myStates.contains(sPresentation)) {
			gc.setColor(Color.RED);
		}
	}
	
	private void drawCondition(ITransition transition, Point point, Graphics gc, double theta) {
		if (myTransitions.contains(transition)) {
			drawWayCondition(getSelectedSymbols(transition), transition.getCondition().toString(), point, gc, theta);
		} else {
			super.drawCondition(transition.getCondition().toString(), point, gc, theta);
		}
	}
	
	private Set<Character> getSelectedSymbols(ITransition transition) {
		Set<Character> symbols = new LinkedHashSet<Character>();
		int index = myTransitions.indexOf(transition);
		if (myTransitions.size() == 1) {
			symbols.add(myString.charAt(index));
			return symbols;
		}
		List<ITransition> tail = myTransitions;
		int shift = 0;
		while (index != - 1 && tail.size() > 0) {
			tail = tail.subList(index + 1, tail.size());
			symbols.add(myString.charAt(index + shift));
			shift += index + 1;
			index = tail.indexOf(transition);
		}
		return symbols;
	}
	
	private void drawWayCondition(Set<Character> symbols, String string, Point point, Graphics gc, double theta) {
		((Graphics2D) gc).rotate( theta, point.getX(), point.getY());
	
		int width = gc.getFontMetrics().stringWidth(string);
		
		int x = point.getX() - width / 2;
		int y = point.getY() - 3;
	
		StringBuilder sb = new StringBuilder();
		int i = 0;
		for (i = 0; i < string.length(); i++) {
			char ch = string.charAt(i);
			if (symbols.contains(ch)) {
				String out = sb.toString();
				gc.drawString(out, x, y);
				sb = new StringBuilder();
				x += gc.getFontMetrics().stringWidth(out); 
				gc.setColor(Color.RED);
				gc.drawString(((Character) ch).toString() , x, y);
				gc.setColor(Color.BLACK);
				x += gc.getFontMetrics().stringWidth(((Character) ch).toString());
			} else {
				sb.append(ch);
			}
		}
		String out = sb.toString();
		gc.drawString(out, x, y);
		
		((Graphics2D) gc).rotate( - theta, point.getX(), point.getY());
	}
	
}
