package ru.amse.bazylevich.faeditor.ui.fautomaton.tools.impl;

import java.awt.Graphics;
import java.awt.event.MouseEvent;

import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Condition;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Transition;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.JFAutomaton;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.util.Geometry;
import ru.amse.bazylevich.faeditor.ui.fautomaton.tools.ITool;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.AutomatonPresentatios;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.Point;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.impl.TransitionParser;

public class AddTransitionState implements ITool {
	
	private final JFAutomaton myJFAutomaton;
	private IStatePresentation myStartState; 
	private Point myEndPoint;
	private Point myStartPoint;
	private Point myDraggedPoint;
	
	public AddTransitionState(JFAutomaton jFAutomaton, IStatePresentation state) {
		myJFAutomaton = jFAutomaton;
		myStartState = state;
	}
	
	@Override
	public void finishDrawing(Graphics gc) {
		if (myStartState != null) {
			myJFAutomaton.getDraw().drawAddingTransition("", myStartPoint, myEndPoint, gc);
		}
	}
	 
	@Override
	public void mouseClicked(MouseEvent event) {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void mouseDragged(MouseEvent event) {
		myJFAutomaton.fireComponentChanged();
		myEndPoint.setX(event.getX() + myEndPoint.getX() - myDraggedPoint.getX());
		myEndPoint.setY(event.getY() + myEndPoint.getY() - myDraggedPoint.getY());
		myDraggedPoint = new Point(event.getX(), event.getY());
	}
	 
	@Override
	public void mouseMoved(MouseEvent event) {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void mousePressed(MouseEvent event) {
		Point start = new Point(myStartState.getX(), myStartState.getY());
		Point end = new Point(event.getX(), event.getY());
		myStartPoint = Geometry.computingEdgeEnd(start, end);
		myEndPoint = Geometry.computingEdgeEnd(end, start);
		myDraggedPoint = new Point(myStartState.getX(), myStartState.getY());
		
	}  
	
	@Override
	public void mouseReleased(MouseEvent event) {
		Point point = new Point(event.getX(), event.getY());
		IStatePresentation newEnd = AutomatonPresentatios.getNearStatePresentation(myJFAutomaton.getAutomatonPresentation(), point);
		if (newEnd != null) {
			boolean duplication = false;
			for (ITransition oldTransition : myStartState.getState().getTransitions()) {
				if (oldTransition.getEnd() == newEnd.getState()) {
					duplication = true;
				}
			}
			if (!duplication) {
				ITransition transition = new Transition(myStartState.getState(), newEnd.getState(), new Condition(""));
				myStartState.getState().addTransition(transition);
				myJFAutomaton.getLabelEditor().setParser(new TransitionParser(transition), point);
				myJFAutomaton.add(myJFAutomaton.getLabelEditor());
				myJFAutomaton.getLabelEditor().requestFocusInWindow();
			}
		}
		myStartState = null;
		myJFAutomaton.repaint();
	}
}
